@@ -1,5 +1,6 @@
 import struct
 import copy
+import uuid
 import os
 import io
 
@@ -10,6 +11,7 @@ from pyroute2.netlink import NLMSG_TRANSPORT
 from pyroute2.netlink import IPRCMD_ACK
 from pyroute2.netlink import IPRCMD_SERVE
 from pyroute2.netlink import IPRCMD_REGISTER
+from pyroute2.netlink import IPRCMD_UNREGISTER
 from pyroute2.netlink import IPRCMD_SHUTDOWN
 from pyroute2.netlink import IPRCMD_CONNECT
 from pyroute2.netlink import IPRCMD_DISCONNECT
@@ -235,6 +237,9 @@ class IOCore(object):
         else:
             return None
 
+    def unregister(self, addr=None):
+        return self.command(IPRCMD_UNREGISTER, addr=addr)
+
     def register(self, secret, addr=None):
         return self.command(IPRCMD_REGISTER,
                             [['IPR_ATTR_SECRET', secret]],
@@ -470,6 +475,7 @@ class IOCore(object):
         envelope['src'] = self.default_broker
         envelope['dport'] = port
         envelope['ttl'] = 16
+        envelope['id'] = uuid.uuid4().bytes
         envelope['attrs'] = [['IPR_ATTR_CDATA', msg]]
         if cname is not None:
             envelope['attrs'].append(['IPR_ATTR_CNAME', cname])
